import { createHash } from 'crypto';
import { linariaLogger } from '@linaria/logger';
import { getFileIdx } from '@linaria/utils';
function hashContent(content) {
  return createHash('sha256').update(content).digest('hex');
}
const cacheLogger = linariaLogger.extend('cache');
const cacheNames = ['entrypoints', 'exports'];
const loggers = cacheNames.reduce((acc, key) => ({
  ...acc,
  [key]: cacheLogger.extend(key)
}), {});
export class TransformCacheCollection {
  contentHashes = new Map();
  constructor(caches = {}) {
    this.entrypoints = caches.entrypoints || new Map();
    this.exports = caches.exports || new Map();
  }
  add(cacheName, key, value) {
    const cache = this[cacheName];
    loggers[cacheName]('%s:add %s %f', getFileIdx(key).toString().padStart(5, '0'), key, () => {
      if (!cache.has(key)) {
        return 'added';
      }
      return cache.get(key) === value ? 'unchanged' : 'updated';
    });
    cache.set(key, value);
  }
  clear(cacheName) {
    if (cacheName === 'all') {
      cacheNames.forEach(name => {
        this.clear(name);
      });
      return;
    }
    loggers[cacheName]('clear');
    const cache = this[cacheName];
    cache.clear();
  }
  delete(cacheName, key) {
    this.invalidate(cacheName, key);
  }
  get(cacheName, key) {
    const cache = this[cacheName];
    const res = cache.get(key);
    loggers[cacheName]('get', key, res === undefined ? 'miss' : 'hit');
    return res;
  }
  has(cacheName, key) {
    const cache = this[cacheName];
    const res = cache.has(key);
    loggers[cacheName]('has', key, res);
    return res;
  }
  invalidate(cacheName, key) {
    const cache = this[cacheName];
    if (!cache.has(key)) {
      return;
    }
    loggers[cacheName]('invalidate', key);
    cache.delete(key);
  }
  invalidateForFile(filename) {
    cacheNames.forEach(cacheName => {
      this.invalidate(cacheName, filename);
    });
  }
  invalidateIfChanged(filename, content) {
    const hash = this.contentHashes.get(filename);
    const newHash = hashContent(content);
    if (hash !== newHash) {
      cacheLogger('content has changed, invalidate all for %s', filename);
      this.contentHashes.set(filename, newHash);
      this.invalidateForFile(filename);
      return true;
    }
    return false;
  }
}
//# sourceMappingURL=cache.js.map